function [DAT] = DRT_mexProp_lensppcomplexIP(aa,Xin,Yin,Toff,Ny,Nt,Nip,xy0,dxy,Vxz,nt)
%% function [DAT] = DRT_mexProp_lensppcomplexIP(aa,Xin,Yin,Toff,Ny,Nt,Nip,xy0,dxy, Vxz, nt)
% dynamic ray tracing from regular grid in x and interpolation of ray field on regular grid in y
% ---input
% aa - takeoff angle
% Xin, Yin - input domain
% Toff - propagation timr
% Ny, Nt - grid size of ray field
% Nip - grid size of interpolation field
% xy0 - origin of interpolation grid
% dxy - resolution of interpolation grid
% Vxz - structure with velocity model
% nt - number of propagation time steps
% ---output
% DAT - structure with dynamic ray tracing and interpolation results

aa=aa-pi/2;

if numel(Nip)~=2; Nip1=Nip(1); Nip2=Nip(1); else; Nip1=Nip(1); Nip2=Nip(2); end

%% --- RAY TRACING
vxz=Vxz.v; dx=Vxz.dx; dz=Vxz.dz; nx=Vxz.nx; nz=Vxz.nz; fx=Vxz.fx; fz=Vxz.fz;
ddt=Toff/nt;ft=0;
yy=linspace(Xin(1),Xin(2),Ny);
zz=linspace(Yin(1),Yin(2),Nt);
[Xds,Yds]=meshgrid(yy,zz); yy0=Xds(:);zz0=Yds(:);
aa0=aa*ones(size(yy0));
dt=ddt*ones(size(yy0));

V0=interp2(Vxz.z',Vxz.x',vxz',Xds,Yds,'cubic');
xip0=linspace(Xin(1),Xin(2),Nip1); yip0=[linspace(Yin(1),Yin(2),Nip2)]; [Xip,Yip]=meshgrid(xip0,yip0);

% ray field tracing (downsampled)
[Yrd]=ray_prop_box_mex(vxz,yy0,zz0,aa0,dt,ft,dx,fx,dz,fz,nx,nz,nt);
Yyz=reshape(Yrd(:,1),[],Ny);
Zzz=reshape(Yrd(:,2),[],Ny);
Tyy=reshape(Yrd(:,3),[],Ny);
Kky=reshape(Yrd(:,6),[],Ny);
Kkz=reshape(Yrd(:,5),[],Ny);
Q1t=reshape(Yrd(:,7),[],Ny);
P1t=reshape(Yrd(:,8),[],Ny);
Q2t=reshape(Yrd(:,9),[],Ny)./V0;
P2t=reshape(Yrd(:,10),[],Ny)./V0;
the_c=reshape(Yrd(:,12),[],Ny);
kmah=reshape(Yrd(:,11),[],Ny);
DAT.Yyz=Yyz; DAT.Zzz=Zzz; DAT.Kky=Kky; DAT.Kkz=Kkz; DAT.Xxx=Xds; DAT.Xxz=Yds;

% find out of model rays
inid=find(Tyy>=max(Tyy(:))-1e-1);boundid=find(Tyy<max(Tyy(:))-1e-1);
if ~isempty(boundid); OUT_M=1; else; OUT_M=0; end
Xds0=Xds;Yds0=Yds;Yyz0=Yyz;Zzz0=Zzz;Kky0=Kky;Kkz0=Kkz;Q1t0=Q1t;Q2t0=Q2t;P1t0=P1t;P2t0=P2t;the_c0=the_c;kmah0=kmah; % keep for sheet separation
MASK=ones(size(Xds0)); MASK(boundid)=0;
[BORD] = find_bound(MASK);
if OUT_M;
    Xds=Xds(inid);Yds=Yds(inid);Yyz=Yyz(inid);Zzz=Zzz(inid);Tyy=Tyy(inid);Kky=Kky(inid);Kkz=Kkz(inid);Q1t=Q1t(inid);Q2t=Q2t(inid);P1t=P1t(inid);P2t=P2t(inid);the_c=the_c(inid); kmah=kmah(inid);
end
xg=Xds0(1,:)';yg=Yds0(:,1);

% check if homogeneous
ssvx=std(vxz(:)); if ssvx>eps; NOR=0; else; NOR=1; end

x1b{1}=[Xds0(BORD.indb{1}(end:-1:1)); Xds0(BORD.indb{3}(1:end)); Xds0(BORD.indb{2}(1:end)); Xds0(BORD.indb{4}(end:-1:1))];
x2b{1}=[Yds0(BORD.indb{1}(end:-1:1)); Yds0(BORD.indb{3}(1:end)); Yds0(BORD.indb{2}(1:end)); Yds0(BORD.indb{4}(end:-1:1))];
y1b{1}=[Yyz0(BORD.indb{1}(end:-1:1)); Yyz0(BORD.indb{3}(1:end)); Yyz0(BORD.indb{2}(1:end)); Yyz0(BORD.indb{4}(end:-1:1))];
y2b{1}=[Zzz0(BORD.indb{1}(end:-1:1)); Zzz0(BORD.indb{3}(1:end)); Zzz0(BORD.indb{2}(1:end)); Zzz0(BORD.indb{4}(end:-1:1))];
y1b{1}=flipud(y1b{1});y2b{1}=flipud(y2b{1});
[XX,YY,indXc{1}] = reg_inpoly_ind(flipud([x1b{1},x2b{1}]),xg,yg); indXc{1}=indXc{1}';
k=1;
x1ipc{k}=[Xds0(indXc{k})];x2ipc{k}=[Yds0(indXc{k})];y1ipc{k}=[Yyz0(indXc{k})];y2ipc{k}=[Zzz0(indXc{k})];q1ipc{k}=[Q1t0(indXc{k})];q2ipc{k}=[Q2t0(indXc{k})];p1ipc{k}=[P1t0(indXc{k})];p2ipc{k}=[P2t0(indXc{k})];vipc{k} =[the_c0(indXc{k})];
ampipc{k}=sqrt(1./q1ipc{k});

%% --- INTERPOLATION OF d2S
indXcs2{2}=1:numel(Xip); indXcs2{1}=[];

dS2_2_xt=[]; X1s=[]; X2s=[];
warning off;
for m=1:length(indXc)
    dS2_2_xt=[dS2_2_xt; q2ipc{m}./q1ipc{m}]; X1s=[X1s; x1ipc{m}]; X2s=[X2s; x2ipc{m}];
end
dS2ip=griddata(X1s,X2s,dS2_2_xt,Xip,Yip,'cubic');
warning on;
DAT.d2S=fliplr(flipud(dS2ip));
DAT.indXcs2=indXcs2;

%% --- INTERPOLATION OF Tvk AND amp
mm1=[];mm2=[];MM1=[];MM2=[];
r1=[]; r2=[]; R1=[]; R2=[];
for m=1:length(indXc)
    [Ysg{m},Zsg{m}] = reg_inpolypp([y1b{m},y2b{m}],xy0,dxy);
    
    xip{m}=griddata(y1ipc{m},y2ipc{m},x1ipc{m},Ysg{m},Zsg{m},'cubic');
    tip{m}=griddata(y1ipc{m},y2ipc{m},x2ipc{m},Ysg{m},Zsg{m},'cubic');
    AMPip{m}=griddata(y1ipc{m},y2ipc{m},real(ampipc{m}),Ysg{m},Zsg{m},'cubic')+1i*griddata(y1ipc{m},y2ipc{m},imag(ampipc{m}),Ysg{m},Zsg{m},'cubic');
    
    % corresponding indices
    iy=round((Ysg{m}-xy0(1))/dxy(1)); iz=round((Zsg{m}-xy0(2))/dxy(2));
    vid=find(~isnan(xip{m})&~isnan(tip{m}));
    xip{m}=xip{m}(vid);AMPip{m}=AMPip{m}(vid);tip{m}=tip{m}(vid);Iy{m}=iy(vid);Iz{m}=iz(vid);
    
    % output region
    mm1=[mm1 min([Ysg{m}]-xy0(1))]; MM1=[MM1 max([Ysg{m}]-xy0(1))];
    mm2=[mm2 min([Zsg{m}]-xy0(2))]; MM2=[MM2 max([Zsg{m}]-xy0(2))];
    r1=[r1 min(Ysg{m})]; r2=[r2 min(Zsg{m})]; R1=[R1 max(Ysg{m})]; R2=[R2 max(Zsg{m})];
end
m1=min(mm1); M1=max(MM1); m1id=floor(m1/dxy(1)); M1id=ceil(M1/dxy(1)); N1=M1id-m1id+1;
m2=min(mm2); M2=max(MM2); m2id=floor(m2/dxy(2)); M2id=ceil(M2/dxy(2)); N2=M2id-m2id+1;
rr1=min(r1); rr2=min(r2); RR1=max(R1); RR2=max(R2);
for m=1:length(indXc)
    DAT.IND{m}=sub2ind([N1,N2],Iy{m}-m1id+1,Iz{m}-m2id+1);
end

%% --- ASSIGN OUTPUT
DAT.X=xip;
DAT.T=tip;
DAT.Y=Ysg;
DAT.Z=Zsg;
DAT.N=[N1 N2];
DAT.ID0=[m1id m2id];
DAT.AMPip=AMPip;
DAT.yrange=[rr1 RR1];
DAT.zrange=[rr2 RR2];
DAT.V0=V0;
DAT.Vt=the_c;
DAT.kmah=kmah;
DAT.x1b=x1b;DAT.x2b=x2b;
DAT.y1b=y1b;DAT.y2b=y2b;


%%