//You can include any C libraries that you normally use
#include "math.h"
#include "mex.h"   //--This one is required


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	//Declarations
	// INPUT
	mxArray *xLP;		// Legendre polys x
	mxArray *zLP;		// Legendre polys z
	mxArray *dJK;	// expansion
	mxArray *ord;	// PSWF max order
	mxArray *sCorr;	// sign correction
	mxArray *EV;	// EV (real / complex)
	// OUTPUT
	double *xPS, *zPSr, *zPSi;  // PSWF
	// INTERN
	double *xlp, *zlp, *djk, *sc, *evr, *evi;
	mwSize xLen, zLen, ordLP, ordPS, i, j, k;
	
	//Copy input pointers
	xLP = prhs[0]; 
	zLP = prhs[1]; 
	dJK = prhs[2];
	ord = prhs[3];
	sCorr = prhs[4];
	EV = prhs[5];
	
	//Get input
	xlp = mxGetPr(xLP);		// legendre x
	zlp = mxGetPr(zLP);		// legendre z
	xLen = mxGetM(xLP);	// x length
	zLen = mxGetM(zLP);	// z length
	djk = mxGetPr(dJK);		// exp coefs pswf
	ordLP = mxGetM(dJK);	// legendre max order
	ordPS = (int)(mxGetScalar(ord)); // PSWF max order
	sc = mxGetPr(sCorr);		// exp coefs pswf
	evr = mxGetPr(EV);		// exp coefs pswf
	evi = mxGetPi(EV);		// exp coefs pswf
	
	//Allocate memory and assign output pointer
	plhs[0] = mxCreateDoubleMatrix(xLen, ordPS, mxREAL); //mxReal is our data-type
	plhs[1] = mxCreateDoubleMatrix(zLen, ordPS, mxCOMPLEX); //mxReal is our data-type
	
	//Get a pointer to the data space in our newly allocated memory
	xPS = mxGetPr(plhs[0]);
	zPSr = mxGetPr(plhs[1]);
	zPSi = mxGetPi(plhs[1]);

	// initialize PSWF
	for(i=0;i<ordPS;i++){ // LOOP PSWF ORDER
		for(j=0;j<xLen;j++){ // LOOP COORDINATE x
			xPS[(i*xLen)+j] = 0.0;
			//*(xPS + (i*xLen)+j) = 0.0;
		}
		for(j=0;j<zLen;j++){ // LOOP COORDINATE z
			zPSr[(i*zLen)+j] = 0.0; zPSi[(i*zLen)+j] = 0.0;
			//*(zPSr+(i*zLen)+j) = 0.0; *(zPSi + (i*zLen)+j) = 0.0;
		}
	}
	
	// compute PSWF
	for(i=0;i<ordPS-1;i++,i++){ // LOOP PSWF ORDER
		for(k=0;k<ordLP;k++){ // LOOP LEGENDRE ORDER
			for(j=0;j<xLen;j++){ // LOOP COORDINATE: x
				xPS[(i*xLen)+j] = xPS[(i*xLen)+j] + sc[i]*xlp[(k*xLen)+j]*djk[(i*ordLP)+k];
				xPS[((i+1)*xLen)+j] = xPS[((i+1)*xLen)+j] + sc[(i+1)]*xlp[(k*xLen)+j]*djk[((i+1)*ordLP)+k];
				//*(xPS+(i*xLen)+j) = *(xPS+(i*xLen)+j) + *(sc+i) * *(xlp+(k*xLen)+j) * *(djk+(i*ordLP)+k);
				//*(xPS+((i+1)*xLen)+j) = *(xPS+((i+1)*xLen)+j) + *(sc+i+1) * *(xlp+(k*xLen)+j) * *(djk+((i+1)*ordLP)+k);
			}
			for(j=0;j<zLen;j++){ // LOOP COORDINATE: x
				zPSr[(i*zLen)+j] = zPSr[(i*zLen)+j] + evr[i]*sc[i]*zlp[(k*zLen)+j]*djk[(i*ordLP)+k];
				zPSi[((i+1)*zLen)+j] = zPSi[((i+1)*zLen)+j] + evi[(i+1)]*sc[(i+1)]*zlp[(k*zLen)+j]*djk[((i+1)*ordLP)+k];
				//*(zPSr+(i*zLen)+j) = *(zPSr+(i*zLen)+j) + *(evr+1) * *(sc+i) * *(zlp+(k*zLen)+j) * *(djk+(i*ordLP)+k);
				//*(zPSi+((i+1)*zLen)+j) = *(zPSi+((i+1)*zLen)+j) + *(evi+i+1) * *(sc+i+1) * *(zlp+(k*zLen)+j) * *(djk+((i+1)*ordLP)+k);
			}
		}
	}
	// case uneven order
	i=ordPS-1;
	for(j=0;j<xLen;j++){xPS[(i*xLen)+j] = 0.0;}
	for(j=0;j<zLen;j++){zPSr[(i*zLen)+j] = 0.0; zPSi[(i*zLen)+j] = 0.0;}
	
	for(k=0;k<ordLP;k++){ // LOOP LEGENDRE ORDER
		for(j=0;j<xLen;j++){ // LOOP COORDINATE: x
			xPS[(i*xLen)+j] = xPS[(i*xLen)+j] + sc[i]*xlp[(k*xLen)+j]*djk[(i*ordLP)+k];
		}
		for(j=0;j<zLen;j++){ // LOOP COORDINATE: x
			zPSr[(i*zLen)+j] = zPSr[(i*zLen)+j] + evr[i]*sc[i]*zlp[(k*zLen)+j]*djk[(i*ordLP)+k];
			zPSi[(i*zLen)+j] = zPSi[(i*zLen)+j] + evi[i]*sc[i]*zlp[(k*zLen)+j]*djk[(i*ordLP)+k];
		}
	}
	


	return;

}


