#include <math.h>
#include "mex.h"

#define PI 3.14159265358979
#define round(x) ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))

double ** newMat(int dim1, int dim2){
	int i, j;
	double  **matrice=(double **)malloc(dim1*sizeof(double *));
	for (i = 0; i < dim1; i++){
		matrice[i] = (double *)malloc(dim2*sizeof(double));
	}
	for (i = 0; i < dim1; i++)
		for (j = 0; j < dim2; j++)
			matrice[i][j] = 0.0;
	return matrice;
}
void deleteMat(int dim1, double **matrice){
	int i;
	for (i = 0; i < dim1; i++)
		free(matrice[i]);
	free(matrice);
}
float ** newMat_s(int dim1, int dim2){
	int i, j;
	float  **matrice=(float **)malloc(dim1*sizeof(float *));
	for (i = 0; i < dim1; i++){
		matrice[i] = (float *)malloc(dim2*sizeof(float));
	}
	for (i = 0; i < dim1; i++)
		for (j = 0; j < dim2; j++)
			matrice[i][j] = 0.0;
	return matrice;
}
void deleteMat_s(int dim1, float **matrice){
	int i;
	for (i = 0; i < dim1; i++)
		free(matrice[i]);
	free(matrice);
}


void Uproj(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,double *Gr, double* Gi)
{
	int	j1,j2,k,l1,l2;	
	double **Er, **Ei,**b;
	double g;
	double a,a1,a2;
	double s,s1,s2,t1,t2;
	int y1,y2;
	double xk1,xk2;
	Er=newMat(2*M_lambda+N1+1,2*M_lambda+N2+1);
	Ei=newMat(2*M_lambda+N1+1,2*M_lambda+N2+1);
	b=newMat(2*M_lambda+1,2*M_lambda+1);
	for(j1=0;j1<2*M_lambda+N1;j1++)
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=0.0;
			Ei[j1][j2]=0.0;
		}
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=exp(-lambda*(j1*j1+j2*j2));
	
	if (Fi!=NULL){
		for (k=0;k<M;k++){
			xk1=2*x1[k];
			xk2=2*x2[k];
			y1=(int)(xk1);
			y2=(int)(xk2);
			a1=exp(-lambda*( (xk1-y1)*(xk1-y1)));
			a2=exp(-lambda*( (xk2-y2)*(xk2-y2)));
			a=a1*a2;
			s1=exp(-lambda*( 2*(xk1-y1)*M_lambda));
			s2=exp(-lambda*( 2*(xk2-y2)*M_lambda));
			t1=exp(lambda*( 2*(xk1-y1)));
			t2=exp(lambda*( 2*(xk2-y2)));
			l1=y1+N1/2;
			for(j1=-M_lambda;j1<=M_lambda;j1++){
				s=s1*s2;
				l2=y2+N2/2;
				for(j2=-M_lambda;j2<=M_lambda;j2++){
					g=a*b[j1+M_lambda][j2+M_lambda]*s;
					Er[l1][l2] += g*Fr[k];
					Ei[l1][l2] += g*Fi[k];
					l2++;
					s*=t2;
				}
				s1*=t1;
				l1++;
			}
		}
	} else {
		for (k=0;k<M;k++){
			xk1=2*x1[k];
			xk2=2*x2[k];
			y1=(int)(xk1);
			y2=(int)(xk2);
			a1=exp(-lambda*( (xk1-y1)*(xk1-y1)));
			a2=exp(-lambda*( (xk2-y2)*(xk2-y2)));
			a=a1*a2;
			s1=exp(-lambda*( 2*(xk1-y1)*M_lambda));
			s2=exp(-lambda*( 2*(xk2-y2)*M_lambda));
			t1=exp(lambda*( 2*(xk1-y1)));
			t2=exp(lambda*( 2*(xk2-y2)));
			l1=y1+N1/2;
			for(j1=-M_lambda;j1<=M_lambda;j1++){
				s=s1*s2;
				l2=y2+N2/2;
				for(j2=-M_lambda;j2<=M_lambda;j2++){
					g=a*b[j1+M_lambda][j2+M_lambda]*s;
					Er[l1][l2] += g*Fr[k];
					l2++;
					s*=t2;
				}
				s1*=t1;
				l1++;
			}
		}
	}  
	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1+N1][j2]+=Er[j1][j2];
			Ei[j1+N1][j2]+=Ei[j1][j2];
		}
	}
	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1+M_lambda][j2]+=Er[j1+M_lambda+N1][j2];
			Ei[j1+M_lambda][j2]+=Ei[j1+M_lambda+N1][j2];
		}
	}
	for(j2=0;j2<M_lambda;j2++){
		for(j1=0;j1<N1;j1++){
			Er[j1+M_lambda][j2+N2]+=Er[j1+M_lambda][j2];
			Ei[j1+M_lambda][j2+N2]+=Ei[j1+M_lambda][j2];
		}
	}
	for(j2=0;j2<M_lambda;j2++){
		for(j1=0;j1<N1;j1++){
			Er[j1+M_lambda][j2+M_lambda]+=Er[j1+M_lambda][j2+N2+M_lambda];
			Ei[j1+M_lambda][j2+M_lambda]+=Ei[j1+M_lambda][j2+N2+M_lambda];
		}
	}
	for(j1=0;j1<N1;j1++){
		for(j2=0;j2<N2;j2++){
			Gr[j1+N1*j2]=lambda/PI*Er[j1+M_lambda][j2+M_lambda];
			Gi[j1+N1*j2]=lambda/PI*Ei[j1+M_lambda][j2+M_lambda];
		}
	}
	deleteMat(2*M_lambda+N1,Er);deleteMat(2*M_lambda+N1,Ei);deleteMat(2*M_lambda+1,b);
}


void Uproj_single_prec(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,float *Gr, float* Gi)
{
	int	j1,j2,k,l1,l2;
	float **Er, **Ei,**b, *fr, *fi;
	float g;
	float a,a1,a2;
	float s,s1,s2,t1,t2;
	int y1,y2;
	float xk1,xk2;
	Er=newMat_s(2*M_lambda+N1+1,2*M_lambda+N2+1);
	Ei=newMat_s(2*M_lambda+N1+1,2*M_lambda+N2+1);
	b=newMat_s(2*M_lambda+1,2*M_lambda+1);
	fr=(float*)malloc(M*sizeof (float));
	fi=(float*)malloc(M*sizeof (float));  
	for(j1=0;j1<2*M_lambda+N1;j1++)
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=0.0;
			Ei[j1][j2]=0.0;
		}
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=(float)exp(-lambda*(j1*j1+j2*j2));
	
	if (Fi!=NULL){
		for (k=0;k<M;k++){
			fr[k]=(float)Fr[k];
			fi[k]=(float)Fi[k];
		}
		for (k=0;k<M;k++){
			xk1=2*(float)x1[k];
			xk2=2*(float)x2[k];
			y1=(int)(xk1);
			y2=(int)(xk2);
			a1=(float)exp(-lambda*( (xk1-y1)*(xk1-y1)));
			a2=(float)exp(-lambda*( (xk2-y2)*(xk2-y2)));
			a=a1*a2;
			s1=(float)exp(-lambda*( 2*(xk1-y1)*M_lambda));
			s2=(float)exp(-lambda*( 2*(xk2-y2)*M_lambda));
			t1=(float)exp(lambda*( 2*(xk1-y1)));
			t2=(float)exp(lambda*( 2*(xk2-y2)));
			l1=y1+N1/2;
			for(j1=-M_lambda;j1<=M_lambda;j1++){
				s=s1*s2;
				l2=y2+N2/2;
				for(j2=-M_lambda;j2<=M_lambda;j2++){
					g=a*b[j1+M_lambda][j2+M_lambda]*s;
					Er[l1][l2] += g*fr[k];
					Ei[l1][l2] += g*fi[k];
					l2++;
					s*=t2;
				}
				s1*=t1;
				l1++;
			}
		}
	} else {
		for (k=0;k<M;k++){
			fr[k]=(float)Fr[k];
		}
		for (k=0;k<M;k++){
			xk1=2*(float)x1[k];
			xk2=2*(float)x2[k];
			y1=(int)(xk1);
			y2=(int)(xk2);
			a1=(float)exp(-lambda*( (xk1-y1)*(xk1-y1)));
			a2=(float)exp(-lambda*( (xk2-y2)*(xk2-y2)));
			a=a1*a2;
			s1=(float)exp(-lambda*( 2*(xk1-y1)*M_lambda));
			s2=(float)exp(-lambda*( 2*(xk2-y2)*M_lambda));
			t1=(float)exp(lambda*( 2*(xk1-y1)));
			t2=(float)exp(lambda*( 2*(xk2-y2)));
			l1=y1+N1/2;
			for(j1=-M_lambda;j1<=M_lambda;j1++){
				s=s1*s2;
				l2=y2+N2/2;
				for(j2=-M_lambda;j2<=M_lambda;j2++){
					g=a*b[j1+M_lambda][j2+M_lambda]*s;
					Er[l1][l2] += g*fr[k];
					l2++;
					s*=t2;
				}
				s1*=t1;
				l1++;
			}
		}
	}  
	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1+N1][j2]+=Er[j1][j2];
			Ei[j1+N1][j2]+=Ei[j1][j2];
		}
	}
	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1+M_lambda][j2]+=Er[j1+M_lambda+N1][j2];
			Ei[j1+M_lambda][j2]+=Ei[j1+M_lambda+N1][j2];
		}
	}
	for(j2=0;j2<M_lambda;j2++){
		for(j1=0;j1<N1;j1++){
			Er[j1+M_lambda][j2+N2]+=Er[j1+M_lambda][j2];
			Ei[j1+M_lambda][j2+N2]+=Ei[j1+M_lambda][j2];
		}
	}
	for(j2=0;j2<M_lambda;j2++){
		for(j1=0;j1<N1;j1++){
			Er[j1+M_lambda][j2+M_lambda]+=Er[j1+M_lambda][j2+N2+M_lambda];
			Ei[j1+M_lambda][j2+M_lambda]+=Ei[j1+M_lambda][j2+N2+M_lambda];
		}
	}
	for(j1=0;j1<N1;j1++){
		for(j2=0;j2<N2;j2++){
			Gr[j1+N1*j2]=(double)(lambda/PI*Er[j1+M_lambda][j2+M_lambda]);
			Gi[j1+N1*j2]=(double)(lambda/PI*Ei[j1+M_lambda][j2+M_lambda]);
		}
	}
	deleteMat_s(2*M_lambda+N1,Er);deleteMat_s(2*M_lambda+N1,Ei);deleteMat_s(2*M_lambda+1,b);free(fr); free(fi);
}

void Vproj(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,double *Gr, double* Gi)
{
	int	j1,j2,k,l1,l2;
	double **Er, **Ei, **b;
	double g;
	double a,a1,a2;
	double s,s1,s2,t1,t2;
	int y1,y2;
	double xk1,xk2;
	double Grk, Gik; 
	Er=newMat(2*M_lambda+N1,2*M_lambda+N2);
	Ei=newMat(2*M_lambda+N1,2*M_lambda+N2);
	b=newMat(2*M_lambda+1,2*M_lambda+1);
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=exp(-lambda*(j1*j1+j2*j2));
	for(j1=M_lambda;j1<M_lambda+N1;j1++){
		for(j2=0;j2<M_lambda;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda+N2)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda+N2)*N1];
		}
		for(j2=M_lambda;j2<M_lambda+N2;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda)*N1];
		}
		for(j2=M_lambda+N2;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda-N2)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda-N2)*N1];
		}
	}
 	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1+N1][j2];
			Ei[j1][j2]=Ei[j1+N1][j2];
		}
	}  
	for(j1=M_lambda+N1;j1<2*M_lambda+N1;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1-N1][j2];
			Ei[j1][j2]=Ei[j1-N1][j2];
		}
	}  
	for (k=0;k<M;k++){
		xk1=2*x1[k];
		xk2=2*x2[k];
		y1=(int)(xk1);
		y2=(int)(xk2);
		a1=exp(-lambda*( (xk1-y1)*(xk1-y1)));
		a2=exp(-lambda*( (xk2-y2)*(xk2-y2)));
		a=a1*a2;
		s1=exp(-lambda*( 2*(xk1-y1)*M_lambda));
		s2=exp(-lambda*( 2*(xk2-y2)*M_lambda));
		t1=exp(lambda*( 2*(xk1-y1)));
		t2=exp(lambda*( 2*(xk2-y2)));
		l1=y1+N1/2;
		Grk=0.0;
		Gik=0.0;
		for(j1=-M_lambda;j1<=M_lambda;j1++){
			s=s1*s2;
			l2=y2+N2/2;
			for(j2=-M_lambda;j2<=M_lambda;j2++){
				g=a*b[j1+M_lambda][j2+M_lambda]*s;
				Grk+=g*Er[l1][l2];
				Gik+=g*Ei[l1][l2];      
				l2++;
				s*=t2;        
			}
			s1*=t1;
			l1++;
		}
		Gr[k]+=Grk*lambda/PI;
		Gi[k]+=Gik*lambda/PI;
	}
	deleteMat(2*M_lambda+N1,Er);deleteMat(2*M_lambda+N1,Ei);deleteMat(2*M_lambda+1,b);
}


void Vproj_single_prec(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,double *Gr, double* Gi)
{
	int	j1,j2,k,l1,l2;
	float **Er, **Ei, **b, *gr, *gi;
	float g;
	float a,a1,a2;
	float s,s1,s2,t1,t2;
	int y1,y2;
	float xk1,xk2;
	float Grk, Gik;
	float lambdas;lambdas=(float)(lambda);
	Er=newMat_s(2*M_lambda+N1,2*M_lambda+N2);
	Ei=newMat_s(2*M_lambda+N1,2*M_lambda+N2);
	b=newMat_s(2*M_lambda+1,2*M_lambda+1);  
	gr=(float*)malloc(M*sizeof (float));
	gi=(float*)malloc(M*sizeof (float));    
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=(float)(exp(-lambdas*(j1*j1+j2*j2)));
	for(j1=M_lambda;j1<M_lambda+N1;j1++){
		for(j2=0;j2<M_lambda;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda+N2)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda+N2)*N1];
		}
		for(j2=M_lambda;j2<M_lambda+N2;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda)*N1];
		}
		for(j2=M_lambda+N2;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda-N2)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda-N2)*N1];
		}
	}
 	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1+N1][j2];
			Ei[j1][j2]=Ei[j1+N1][j2];
		}
	}  
	for(j1=M_lambda+N1;j1<2*M_lambda+N1;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1-N1][j2];
			Ei[j1][j2]=Ei[j1-N1][j2];
		}
	}  
	for (k=0;k<M;k++){
		xk1=2*(float)x1[k];
		xk2=2*(float)x2[k];
		y1=(int)(xk1);
		y2=(int)(xk2);
		a1=(float)exp(-lambdas*( (xk1-y1)*(xk1-y1)));
		a2=(float)exp(-lambdas*( (xk2-y2)*(xk2-y2)));
		a=a1*a2;
		s1=(float)exp(-lambdas*( 2*(xk1-y1)*M_lambda));
		s2=(float)exp(-lambdas*( 2*(xk2-y2)*M_lambda));
		t1=(float)exp(lambdas*( 2*(xk1-y1)));
		t2=(float)exp(lambdas*( 2*(xk2-y2)));
		l1=y1+N1/2;
		Grk=0.0;
		Gik=0.0;
		for(j1=-M_lambda;j1<=M_lambda;j1++){
			s=s1*s2;
			l2=y2+N2/2;
			for(j2=-M_lambda;j2<=M_lambda;j2++){
				g=a*b[j1+M_lambda][j2+M_lambda]*s;
				Grk+=g*Er[l1][l2];
				Gik+=g*Ei[l1][l2];      
				l2++;
				s*=t2;        
			}
			s1*=t1;
			l1++;
		}
		gr[k]=Grk*lambdas/((float)PI);
		gi[k]=Gik*lambdas/((float)PI);
	}
	for(k=0;k<M;k++){
		Gr[k]+=(double)(gr[k]);
		Gi[k]+=(double)(gi[k]);
	}
	deleteMat_s(2*M_lambda+N1,Er);deleteMat_s(2*M_lambda+N1,Ei);deleteMat_s(2*M_lambda+1,b);free(gr);free(gi);
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[] )
{ 
	int N1,N2, M, M_lambda,j1,j2,M2,Md, k1, k2,id,i1,i2;
	double lambda,g,epsilon;
	double *x1,*x2,*x1o,*x2o,*xi1,*xi2;
	double *Fr,*Fi,*Gr,*Gi;
	double *lr,*li,*dr,*di,*er,*ei,*fr,*fi,*ar,*ai;
	float *ars,*ais,*frs,*fis;
	double qr,qi;
	int Mout;
	mxArray* d[1];
	mxArray* e[1];
	mxArray* f[1];
	mxArray* a[1];
	int lowmem, singprec;
	
	M=(int)(mxGetM(prhs[0])*mxGetN(prhs[0]));
	M2=(int)(mxGetM(prhs[1])*mxGetN(prhs[1]));
	Md=(int)(mxGetM(prhs[2])*mxGetN(prhs[2]));  
	if (nrhs <5) { 
		mexErrMsgTxt("Input format: [xi1, xi2, data, N1, N2]"); 
	} else if ( (M!=M2)|(M2!=Md)|(Md!=M)) {
		mexErrMsgTxt("Inconsistent size of positions and data"); 
	} 
	x1o=mxGetPr(prhs[0]);
	x2o=mxGetPr(prhs[1]);
	Fr = mxGetPr(prhs[2]);
	Fi = mxGetPi(prhs[2]);
	x1=(double *)malloc(M*sizeof(double));
	x2=(double *)malloc(M*sizeof(double));
	N1=(int)(mxGetPr(prhs[3]))[0];
	N2=(int)(mxGetPr(prhs[4]))[0];
	
	if (nrhs<8) epsilon=1e-12; else epsilon=(mxGetPr(prhs[7]))[0];
	if (nrhs<9) lowmem=0; else lowmem=(mxGetPr(prhs[8]))[0];
	if (epsilon<1e-7) singprec=0; else singprec=1;
	for (j1=0;j1<M;j1++){
		x1[j1]=x1o[j1]-round(x1o[j1]/(N1))*(N1);
		x2[j1]=x2o[j1]-round(x2o[j1]/(N2))*(N2);
	}  
	lambda=9.86960440108936/(-2*log(epsilon));
	M_lambda=(int)(sqrt(-log(epsilon)/lambda));
	Mout=mxGetM(prhs[5])*mxGetN(prhs[5]);
	xi1=mxGetPr(prhs[5]);xi2=mxGetPr(prhs[6]);  
	plhs[0] = mxCreateDoubleMatrix(Mout,1, mxCOMPLEX);
	Gr=mxGetPr(plhs[0]);Gi=mxGetPi(plhs[0]);	for(j1=0;j1<Mout;j1++){ Gr[j1]=0;Gi[j1]=0;}
	
	if (lowmem==0){
		if(singprec==0){
			a[0]=mxCreateNumericMatrix(2*N2,2*N1, mxDOUBLE_CLASS, mxCOMPLEX);
			ar=mxGetPr(a[0]);ai=mxGetPi(a[0]);
			Uproj(2*N2,2*N1,lambda,M_lambda,M,x2,x1,Fr,Fi,ar,ai);
		}
		else{
			a[0]=mxCreateNumericMatrix(2*N2,2*N1, mxSINGLE_CLASS, mxCOMPLEX);
			ars=mxGetPr(a[0]);ais=mxGetPi(a[0]);
			Uproj_single_prec(2*N2,2*N1,lambda,M_lambda,M,x2,x1,Fr,Fi,ars,ais);
		}
	}
	
	for(k1=0;k1<2;k1++)
		for(k2=0;k2<2;k2++){
			e[0] = mxCreateDoubleMatrix(N2,N1, mxCOMPLEX);
			f[0] = mxCreateDoubleMatrix(N2,N1, mxCOMPLEX);
			fr = mxGetPr(f[0]);fi = mxGetPi(f[0]);
			frs = mxGetPr(f[0]);fis = mxGetPi(f[0]); 
			d[0] = mxCreateDoubleMatrix(N2,N1, mxCOMPLEX);
			dr = mxGetPr(d[0]);di = mxGetPi(d[0]);for(j1=0;j1<N1*N2;j1++){di[j1]=0;dr[j1]=0;}      
			for(i1=0;i1<2;i1++)
				for(i2=0;i2<2;i2++){
					er = mxGetPr(e[0]);ei = mxGetPi(e[0]);
					if(singprec==0){
						if (lowmem==1){
							for (j1=0;j1<M;j1++){x1[j1]=x1[j1]*0.5-.25*i1; x2[j1]=x2[j1]*0.5-.25*i2;};
							Uproj(N2,N1,lambda*4,M_lambda/2,M,x2,x1,Fr,Fi,fr,fi);
							for (j1=0;j1<M;j1++){x1[j1]=2*(x1[j1]+.25*i1); x2[j1]=2*(x2[j1]+.25*i2);};
							for(j1=0;j1<N1;j1++)
								for(j2=0;j2<N2;j2++)
								{
									g=4*exp(9.86960440108936*(((2*j1+i1)-2*N1/2)*((2*j1+i1)-2*N1/2)/(double)(1*N1*1*N1)+((2*j2+i2)-2*N2/2)*((2*j2+i2)-2*N2/2)/(double)(1*N2*1*N2))/(lambda*4*4));
									qr= cos(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									qi=-sin(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									er[j2+j1*N2]=g*(qr*fr[j2+j1*N2]-qi*fi[j2+j1*N2])/4;
									ei[j2+j1*N2]=g*(qi*fr[j2+j1*N2]+qr*fi[j2+j1*N2])/4;              
								}
						}
						else{
							for(j1=0;j1<N1;j1++)
								for(j2=0;j2<N2;j2++)
								{ 
									g=4*exp(9.86960440108936*(((2*j1+i1)-2*N1/2)*((2*j1+i1)-2*N1/2)/(double)(1*N1*1*N1)+((2*j2+i2)-2*N2/2)*((2*j2+i2)-2*N2/2)/(double)(1*N2*1*N2))/(lambda*4*4));
									qr= cos(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									qi=-sin(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									id=(2*j2+i2)+2*N2*(2*j1+i1);
									er[j2+j1*N2]=g*(qr*ar[id]-qi*ai[id]);
									ei[j2+j1*N2]=g*(qi*ar[id]+qr*ai[id]);              
								}
						}
					}else
						if (lowmem==1){
							for (j1=0;j1<M;j1++){x1[j1]=x1[j1]*0.5-.25*i1; x2[j1]=x2[j1]*0.5-.25*i2;};
							Uproj_single_prec(N2,N1,lambda*4,M_lambda/2,M,x2,x1,Fr,Fi,frs,fis);
							for (j1=0;j1<M;j1++){x1[j1]=2*(x1[j1]+.25*i1); x2[j1]=2*(x2[j1]+.25*i2);};
							for(j1=0;j1<N1;j1++)
								for(j2=0;j2<N2;j2++)
								{
									g=4*exp(9.86960440108936*(((2*j1+i1)-2*N1/2)*((2*j1+i1)-2*N1/2)/(double)(1*N1*1*N1)+((2*j2+i2)-2*N2/2)*((2*j2+i2)-2*N2/2)/(double)(1*N2*1*N2))/(lambda*4*4));
									qr= cos(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									qi=-sin(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									er[j2+j1*N2]=g*(qr*frs[j2+j1*N2]-qi*fis[j2+j1*N2])/4;
									ei[j2+j1*N2]=g*(qi*frs[j2+j1*N2]+qr*fis[j2+j1*N2])/4;              
								}
						}
						else{
							for(j1=0;j1<N1;j1++)
								for(j2=0;j2<N2;j2++)
								{ 
									g=4*exp(9.86960440108936*(((2*j1+i1)-2*N1/2)*((2*j1+i1)-2*N1/2)/(double)(1*N1*1*N1)+((2*j2+i2)-2*N2/2)*((2*j2+i2)-2*N2/2)/(double)(1*N2*1*N2))/(lambda*4*4));
									qr= cos(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									qi=-sin(6.283185307179586*(k1*((2*j1+i1)-N1)/(double)(4*N1)+k2*((2*j2+i2)-N2)/(double)(4*N2)));
									id=(2*j2+i2)+2*N2*(2*j1+i1);
									er[j2+j1*N2]=g*(qr*ars[id]-qi*ais[id]);
									ei[j2+j1*N2]=g*(qi*ars[id]+qr*ais[id]);              
								}
						}
					/* ... HERE ARE THE FFTSHIFTS ... */
					mexCallMATLAB(1,f,1, e, "fftshift");
					mexCallMATLAB(1,e,1, f, "fft2");
					mexCallMATLAB(1,f,1, e, "fftshift");
					fr = mxGetPr(f[0]);fi = mxGetPi(f[0]);
					for (j1=0;j1<N1;j1++) 
						for (j2=0;j2<N2;j2++){
							qr= cos(6.283185307179586*(i1*(j1-N1/2.0)/(double)(2.0*N1)+i2*(j2-N2/2.0)/(double)(2.0*N2)));
							qi=-sin(6.283185307179586*(i1*(j1-N1/2.0)/(double)(2.0*N1)+i2*(j2-N2/2.0)/(double)(2.0*N2)));
							dr[j2+N2*j1]+=(qr*fr[j2+N2*j1]-qi*fi[j2+N2*j1]);
							di[j2+N2*j1]+=(qi*fr[j2+N2*j1]+qr*fi[j2+N2*j1]);
						}
				}
			for (j1=0;j1<Mout;j1++){xi1[j1]=xi1[j1]*0.5-.25*k1; xi2[j1]=xi2[j1]*0.5-.25*k2;};
			if(singprec==0)
				Vproj(N2,N1,4*lambda,(int)(M_lambda/2),Mout,xi2,xi1,dr,di,Gr,Gi);
			else
				Vproj_single_prec(N2,N1,4*lambda,(int)(M_lambda/2),Mout,xi2,xi1,dr,di,Gr,Gi);
			for (j1=0;j1<Mout;j1++){xi1[j1]=2*(xi1[j1]+.25*k1); xi2[j1]=2*(xi2[j1]+.25*k2);};
			
		} 
	for(j1=0;j1<Mout;j1++){
		g=0.25*exp(9.86960440108936*(xi1[j1]*xi1[j1]/(double)(N1*N1)+xi2[j1]*xi2[j1]/(double)(N2*N2))/(lambda*4));
		Gr[j1]*=g*.25;Gi[j1]*=g*0.25;
	}
	mxDestroyArray(d[0]);mxDestroyArray(e[0]);mxDestroyArray(f[0]);
	free(x1);
	free(x2);
	return;
}

